/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klarlvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/**
 * RES file writer (ListView backend).
 *
 * Header file for class QtListViewWriter.
 */

#ifndef __QTLISTVIEWWRITER_H__
#define __QTLISTVIEWWRITER_H__

#include <qmap.h>
#include <qstrlist.h>

#include <ResFile.h>

typedef QMap<ushort,QString> ResourceMapByID;
typedef QMap<QString,QString> ResourceMapByString;

class QListView;
class QListViewItem;
class QFile;
class QIODevice;
class ControlData;
class QWidget;

class QtListViewWriter
{
public:
  QtListViewWriter( QWidget* parent = 0 );

  QListView* write( const ResourceList* resources );

private:
  void fixItemText( QString& text, int& accel );

  void recordCursorResource( Resource* );
  void recordBitmapResource( Resource* );
  void recordIconResource( Resource* );
  void recordMenuResource( Resource* );
  void recordDialogResource( Resource* );
  void recordStringResource( Resource* );
  void recordFontDirResource( Resource* );
  void recordFontResource( Resource* );
  void recordAcceleratorsResource( Resource* );
  void recordRCDataResource( Resource* );
  void recordMessageTableResource( Resource* );
  void recordGroupCursorResource( Resource* );
  void recordGroupIconResource( Resource* );
  void recordVersionResource( Resource* );
  void recordNewBitmapResource( Resource* );
  void recordNewMenuResource( Resource* );
  void recordNewDialogResource( Resource* );
  void recordToolbarResource( Resource* );
  void recordDlgInitResource( Resource* );
  void recordUserDefinedResource( Resource* );

  QListView* listview;
  QListViewItem* previousitem;

  QStrList cursorlist;
  QStrList bitmaplist;
  ResourceMapByID menulist1;
  ResourceMapByString menulist2;
  ResourceMapByID dialoglist1;
  ResourceMapByString dialoglist2;
  QStrList stringlist;
  QStrList fontdirlist;
  QStrList fontlist;
  QStrList accellist;
  QStrList toolbarlist;
};

#endif
