/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klarlvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/**
 * RES file writer (Qt Designer XML backend).
 *
 * Header file for class QtDesignerWriter.
 */

#ifndef __QTDESIGNERWRITER_H__
#define __QTDESIGNERWRITER_H__

#include <qmap.h>
#include <qstrlist.h>
#include <qtextstream.h>
#include <qvaluelist.h>
#include <qprogressdialog.h>

#include <qapplication.h>

#include <ResFile.h>
#include <typedefs.h>

typedef QMap<ushort,QString> ResourceMapByID;
typedef QMap<QString,QString> ResourceMapByString;

class QIODevice;
struct ControlData;
class FntMatchEntry;
class CtrlMatchEntry;

struct SearchedIcon
{
  nameorordinal name;
  int cx;
  int cy;

  SearchedIcon() {}
  SearchedIcon( const SearchedIcon& sc ) {
	name = sc.name;
	cx = sc.cx;
	cy = sc.cy;
  }
  SearchedIcon& operator=( const SearchedIcon& sc ) {
	if( this == &sc )
	  return *this;

	name = sc.name;
	cx = sc.cx;
	cy = sc.cy;

	return *this;
  }
};

class QtDesignerWriter
{
public:
  QtDesignerWriter( const QString& filename, float scalefactor,
                    const QString& rcfilename, QProgressDialog *pdlg );
  QtDesignerWriter( QIODevice* file,         float scalefactor,
                    const QString& rcfilename, QProgressDialog *pdlg );
  ~QtDesignerWriter();

  bool write( const ResourceList* resources );

private:
  void recordCursorResource( Resource* );
  void recordBitmapResource( Resource* );
  void recordIconResource( Resource* );
  void recordMenuResource( Resource* );
  void recordDialogResource( QTextStream&, Resource* );
  void recordStringResource( Resource* );
  void recordFontDirResource( Resource* );
  void recordFontResource( Resource* );
  void recordAcceleratorsResource( Resource* );
  void recordRCDataResource( Resource* );
  void recordMessageTableResource( Resource* );
  void recordGroupCursorResource( Resource* );
  void recordGroupIconResource( Resource* );
  void recordVersionResource( Resource* );
  void recordNewBitmapResource( Resource* );
  void recordNewMenuResource( Resource* );
  void recordNewDialogResource( Resource* );
  void recordToolbarResource( Resource* );
  void recordDlgInitResource( Resource* );

  void recordImagesForDialog( QTextStream&, Resource* );

  void findGroupScanClassnameGetBuddy(void* controls, ControlData* pGroup);
  void writeControl( QTextStream& s, ControlData* cd,
                     void* controls, ControlData* pGroup );
  QString expandSpecialChars( const QString& );

  void InitFontAndCtrlMatcher( const QString& inifilename );
  bool GetFontMatch( QString font, QString& rOurName, float& rOurMult );
  bool GetCtrlMatch( QString font, WORD& rOurId, Q_UINT32& rOurStyle );

  int scale(   int value ); // multiply by scalefactor
  int unscale( int value ); // divide by scalefactor

  QIODevice* iodevice;
  bool filecreated;
  float scalefactor;  // see scale() and unscale() resp.

  int nStorePlaceholder;    // unknown user def. control classes found?

  int     nOurDlgFontSize;  // needed for guessing height of ComboBoxes
  QString sOurDlgFontName;  // needed for breaking lines on QPushButtons

  QProgressDialog* dlg;

  QStrList cursorlist;
  QStrList bitmaplist;
  ResourceMapByID menulist1;
  ResourceMapByString menulist2;
  ResourceMapByID dialoglist1;
  ResourceMapByString dialoglist2;
  QStrList stringlist;
  QStrList fontdirlist;
  QStrList fontlist;
  QStrList accellist;
  QStrList toolbarlist;

  QValueList<SearchedIcon> imagesfordialog;
  QList<FntMatchEntry>     FontMatcher; // replace fonts by other fonts
  QList<CtrlMatchEntry>    CtrlMatcher; // replace user defined controlls
};

#endif
