/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klarlvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/****************************************************************************
**
** This class is the general Output Wrapper of the application.
** It is called by the main application and any objects that wants to
** write comments to the text screen and/or to log files and/or to the GUI
** It calls 
**
*****************************************************************************/

#include "reswrite.h"

#include <qmultilineedit.h>
#include <qfile.h>
#include <qdatetime.h> 
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qdatastream.h>
#include <qtextstream.h>
#include <iostream>

using namespace std;


MyOut::MyOut(	QWidget*		_parent,
				QString			_sLog,
				QMultiLineEdit*	_pEd,
				QStatusBar*		_pStat,
				bool			_bStdOut,
				bool			_bStdErr ) :
	parent( _parent ),
	pEd( _pEd ), pStat( _pStat ), pfLog( 0 ), pLog( 0 ),
	bStdOut( _bStdOut ),
	bStdErr( _bStdErr ),
	bIOError( FALSE )
{
	if( _sLog.length() )
	{
		SetStat( "Storing output in log file " + _sLog );
	
		pfLog = new QFile( _sLog );
		if( !pfLog->open( IO_WriteOnly | IO_Append )
			|| (IO_Ok != pfLog->status()) )
		{
			bIOError = TRUE;
			return;
		}
		pLog = new QTextStream( pfLog );
		AddLog( "###### --------------------------------------------- "
				"#################\n"
				"###### opening log file at "
				+ QDate::currentDate().toString() + ", "
				+ QTime::currentTime().toString() + " #################\n\n" );
	}
}


MyOut::~MyOut()
{
	if( pfLog )
	{
		AddLog( "\n###### closing log file at "
				+ QDate::currentDate().toString() + ", "
				+ QTime::currentTime().toString() + " #################\n" );
		pLog->unsetDevice();
		pfLog->close();
		delete pfLog;
		delete pLog;
	}	
}


bool MyOut::AddLog( QString sMsg )
{
	bool bResult = TRUE;
	if( bStdOut )
		printf( "%s", sMsg.local8Bit().data() );
	if( bStdErr )
		qDebug( sMsg );
	if( pLog )
	{
		*pLog << sMsg;
		if( IO_Ok != pLog->device()->status() )
			bResult = FALSE;
	}
	if( pEd )
		pEd->append( sMsg );
	return bResult;
}


bool MyOut::SetStat( QString sMsg )
{
	if( pStat )
		pStat->message( sMsg, 2000 );
	return AddLog( "\n\n* * *  " + sMsg + "  * * *\n\n\n" );
}


