/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klarlvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/****************************************************************************
**
** This class is the Ressource File Writer part of the Res To X application.
** It is called by the main application.
** It calls internal class ResDirectory to transmit ressources to .RES file.
**
*****************************************************************************/

#ifndef RESWRITE_H
#define RESWRITE_H

#include <qobject.h>
#include <qprogressdialog.h>

#include <qapplication.h>

#include "myout.h"

class QWidget;
class QString;
class QDataStream;


enum ResWriteResult { RWOk,			// no errors, ALL ressources converted
					RWUserParameterErr,		// parameters invalid/missing
					RWSomeUnimplemented,	// format Ok, but no conversion
					RWSomeUnknown,			// type of ressource unknown
					RWSrcFormatErr,			// Syntax Error in .exe file
					RWSrcIOErr,				// IO Error reading .exe file
					RWDestIOErr,			// IO Error writing .res file
					RWLogIOErr,				// IO Error writing .log file
					RWInternalProgErr };	// Error in our program!  :-(


class ResWrite: public QObject
{
	Q_OBJECT
public:
    ResWrite(QWidget*         _parent,
             QIODevice*       _exedata,
             QIODevice*       _resdata,
             MyOut&           _rOut,
             ResWriteResult&  _rResult,
             QProgressDialog* _dlg );
    ~ResWrite();
QString GetResultTxt( ResWriteResult result );

//protected:

//private slots:

private:
	bool SrcError( bool bError, QString sMsg );
	void DumpPE(	QDataStream&	rSrc,
					QIODevice&		_resdata,
					Q_UINT32		nOffsHd,
					ResWriteResult&	rResult );
	void DumpNE();
	void DumpSE();

private:
    QWidget*         parent;
    MyOut&           rOut;
    ResWriteResult&  rResult;
    QProgressDialog* dlg;
};


#endif
