/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klarlvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/**
 * RES file reader.
 *
 * Header file for class ResBitmap.
 */

#ifndef __RESBITMAP_H__
#define __RESBITMAP_H__

#include "ResData.h"

#include <qimage.h>


#define BFT_BMAP          0x4D42 // 'BM' for bit maps
#define BFT_ICON          0x4349 // 'IC' for icons
#define BFT_POINTER       0x5450 // 'PT' for pointers
#define BFT_COLORICON     0x4943 // 'CI' for color icons
#define BFT_COLORPOINTER  0x5043 // 'CP' for color pointers

#define	BCA_UNCOMP		0x00000000L
#define	BCA_RLE8		0x00000001L
#define	BCA_RLE4		0x00000002L
#define	BCA_HUFFMAN1D	0x00000003L
#define	BCA_RLE24		0x00000004L


struct BITMAPINFOHEADER
{
  Q_UINT32 SizeHd;		// size of this header
  Q_UINT32 Width;		// width of image in Pixels
  Q_UINT32 Height;
  Q_UINT16 Planes;		// bit planes used
  Q_UINT16 BitCount;	// bits per pixel: 1, 4, 8 or 24
  Q_UINT32 Compression; // type of RLE encoding...
  Q_UINT32 SizeImage;
  Q_UINT32 XPelsPerMeter;
  Q_UINT32 YPelsPerMeter;
  Q_UINT32 ColorsUsed;
  Q_UINT32 ColorsImportant;
  
  Q_UINT16 Units;
  Q_UINT16 Reserved;
  Q_UINT16 Recording;
  Q_UINT16 Rendering;
  Q_UINT32 Size1;
  Q_UINT32 Size2;
  Q_UINT32 ColorEncoding;
  Q_UINT32 Identifier;
  
  // after this follows the ColorTable
  // after this follows the BitmapData (at offset BITMAPFILEHEADER.offBits)
};


class ResBitmap : public ResData
{
  friend QDataStream& operator>>( QDataStream& s, ResBitmap& r );

public:
  virtual ~ResBitmap() {}
  
  void read_DIB( Q_UINT16 nBFT, QDataStream& s );
  QString GetCompressionTxt() const;
  virtual QString toString() const;
  QImage image() { return _image; }
  void setImage( QImage & aImage ) { _image = aImage; }
  
  BITMAPINFOHEADER ms; // Microsoft data (for BMP and ICONs)

private:
  QImage _image;
  Q_UINT16 _restype;  // e.g. BFT_BMAP or BFT_ICON, set by read_DIB()
};

QDataStream& operator>>( QDataStream& s, ResBitmap& r );

#endif
