/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klar�lvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/**
 * RES file reader.
 *
 * Header file for class ResDialog.
 */

#ifndef __RESDIALOG_H__
#define __RESDIALOG_H__

#include "ResData.h"
#include "typedefs.h"

#include <qdatastream.h>
#include <qptrlist.h>//<qlist.h>

#define DS_SETFONT 0x40L

#define BUTTON    0x80
#define EDIT      0x81
#define STATIC    0x82
#define LISTBOX   0x83
#define SCROLLBAR 0x84
#define COMBOBOX  0x85

#define SS_ICON   0x00000003L

#define BS_PUSHBUTTON 0x00000000L
#define BS_DEFPUSHBUTTON 0x00000001L
#define BS_CHECKBOX 0x00000002L
#define BS_AUTOCHECKBOX 0x00000003L
#define BS_RADIOBUTTON 0x00000004L
#define BS_3STATE 0x00000005L
#define BS_AUTO3STATE 0x00000006L
#define BS_GROUPBOX 0x00000007L
#define BS_AUTORADIOBUTTON 0x00000009L


#define ES_LEFT   0x0000L
#define ES_RIGHT  0x0001L
#define ES_CENTER 0x0002L

struct ControlData {
  ControlData* myGroup;
  Q_UINT32 lHelpId;
  Q_UINT32 lExtendedStyle;
  Q_UINT32 lStyle;
  Q_UINT16 x;
  Q_UINT16 y;
  Q_UINT16 cx;
  Q_UINT16 cy;
  Q_UINT16 wId;
  nameorordinal classId;
  nameorordinal text;
  Q_UINT16 nExtraStuff;
  int nPlaceholder;
  ControlData(): myGroup( 0 ), nPlaceholder( 0 ) {}
};


struct DialogBoxHeader {
  bool bIsExtended; // header is extended if 1st words are: 0x0001 and 0xffff
  Q_UINT32 lHelpId; // extended header only
  Q_UINT32 lStyle;
  Q_UINT32 lExtendedStyle;
  Q_UINT16 numberOfItems;
  Q_UINT16 x;
  Q_UINT16 y;
  Q_UINT16 cx;
  Q_UINT16 cy;
  nameorordinal menuName;
  nameorordinal className;
  QString caption;
  Q_UINT16 wPointSize;
  Q_UINT16 wWeight; // extended header only
  Q_UINT8  bItalic; // dito
  Q_UINT8  bCharset;// dito
  QString fontName;
};


class ResDialog : public ResData
{
  friend QDataStream& operator>>( QDataStream& s, ResDialog& r );

public:
  ResDialog() {
	_controls.setAutoDelete( true );
  }
  virtual ~ResDialog() {}

  virtual QString toString() const;
  const DialogBoxHeader* header() const { return &dlgheader; }
  QList<ControlData>* controls() { return &_controls; }

private:
  DialogBoxHeader dlgheader;

  QList<ControlData> _controls;
};

QDataStream& operator>>( QDataStream& s, ResDialog& r );

#endif
