/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klarlvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/**
 * RES file reader.
 *
 * Implementation file for class ResFile.
 */

#include <qdatastream.h>
#include <qfile.h>

#include "ResFile.h"
#include "typedefs.h"

ResFile::ResFile( QIODevice* iodevice )
{
  this->iodevice = iodevice;
  reslist.setAutoDelete( true );
}


bool ResFile::read()
{
  if( iodevice->isOpen() ) {
	QDataStream s( iodevice );
	s.setByteOrder( QDataStream::LittleEndian );

	// Skip the first eight bytes.
	// PENDING(kalle) Use this to distinguish 16/32 bit resources.
	short dummy;
	s >> dummy;
	s >> dummy;
	s >> dummy;
	s >> dummy;
	
	// Skip first invalid resource.
	DWORD ldummy;
	s >> ldummy;
	s >> ldummy;
	s >> dummy;
	s >> dummy;
	s >> dummy;
	s >> dummy;
	s >> ldummy;
	s >> ldummy;

	while( !s.atEnd() ) {
	  Resource* res = new Resource();
	  s >> *res;
	  reslist.append( res );
	}
	return true;
  }

  return false;
}


const ResourceList* ResFile::resources() const
{
  return &reslist;
}
