/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klarlvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/**
 * RES file reader.
 *
 * Implementation file for class ResGroupIcon.
 */

#include <math.h>
#include <stdlib.h>

#include "ResGroupIcon.h"
#include "ResHeader.h"

#include <qdatastream.h>

QDict<ResGroupIcon> ResGroupIcon::objects;

ResGroupIcon::ResGroupIcon( ResHeader* header )
{
  _iconentries.setAutoDelete( true );

  // Register object in object table
  objects.insert( header->name().toString(), this );
}

QString ResGroupIcon::toString() const
{
  qDebug( "Sorry, not implemented: ResGroupIcon::toString()" );
  return QString::null;
}


QDataStream& operator>>( QDataStream& s, ResGroupIcon& r )
{
  // Note: The wPadding field that the MS docs specify does not seem
  // to exist.

  // Read the header
  s >> r._iconHeader.wReserved;
  ASSERT( r._iconHeader.wReserved == 0 );
  s >> r._iconHeader.wType;
  ASSERT( r._iconHeader.wType == 1 ); // 1 == icons
  s >> r._iconHeader.wCount;

  // Now read the directory entries one by one
  for( int entry = 0; entry < r._iconHeader.wCount; entry++ ) {
	ResourceDirectory* newentry = new ResourceDirectory;
	s >> newentry->bWidth >> newentry->bHeight >> newentry->bColorCount
	  >> newentry->bReserved >> newentry->wPlanes >> newentry->wBitCount
	  >> newentry->lBytesInRes >> newentry->wNameOrdinal;
	r._iconentries.append( newentry );
  }

  return s;
}


ResGroupIcon* ResGroupIcon::findByNameOrOrdinal( const nameorordinal& name )
{
  return objects.find( name.toString() );
}


static int sizedist( ResourceDirectory* r, int cx, int cy )
{
  return abs( ( r->bWidth - cx ) * ( r->bWidth - cx ) ) +
	abs( ( r->bHeight - cy ) * ( r->bHeight - cy ) );
}


QString ResGroupIcon::findBestIcon( int cx, int cy )
{
  // Algorithm for finding the best icon:
  // First priority is the size: The icon with the shortest sum of
  // size difference squares wins.
  // Second: The icon with the highes number of colors wins.
  
  // Boundary case: No icons in group.
  if( _iconentries.count() == 0 )
	return QString::null;

  ResourceDirectory* best = _iconentries.first();
  int bestdist = sizedist( best, cx, cy );
  QListIterator<ResourceDirectory> it( _iconentries );
  // forward one, no need to compare the first with the first
  ++it;
  ResourceDirectory* current;
  while( ( current = it.current() ) ) {
	++it;

	int currentdist = sizedist( current, cx, cy );
	if( ( bestdist > currentdist ) || // current is closer
		( ( bestdist == currentdist ) &&
		  current->bColorCount > best->bColorCount ) ) { // same distance, but
		// current has more colors
	  best = current;
	  bestdist = currentdist;
	}
  }
  
  // Not sure whether this is correct.
  return QString::number( best->wNameOrdinal );
}


