/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klar�lvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/**
 * RES file reader.
 *
 * Header file for class ResGroupIcon.
 */

#ifndef __RESGROUPICON_H__
#define __RESGROUPICON_H__

#include "ResData.h"
#include "typedefs.h"

class ResHeader;

#include <qdict.h>
#include <qptrlist.h>//<qlist.h>

// This is from MS documentation.
struct IconHeader
{
  Q_UINT16 wReserved; // currently zero
  Q_UINT16 wType; // 1 for icons
  Q_UINT16 wCount; // number of components;
};


struct ResourceDirectory
{
  Q_UINT8 bWidth;
  Q_UINT8 bHeight;
  Q_UINT8 bColorCount;
  Q_UINT8 bReserved;
  Q_UINT16 wPlanes;
  Q_UINT16 wBitCount;
  Q_UINT32 lBytesInRes;
  Q_UINT16 wNameOrdinal; // Points to component
};

int operator<( const ResourceDirectory&, const ResourceDirectory& );
int operator==( const ResourceDirectory&, const ResourceDirectory& );

class ResGroupIcon : public ResData
{
  friend QDataStream& operator>>( QDataStream& s, ResGroupIcon& r );

public:
  ResGroupIcon( ResHeader* header );
  virtual ~ResGroupIcon() {}
  virtual QString toString() const;
  IconHeader* iconHeader() { return &_iconHeader; }
  QList<ResourceDirectory>* iconList() { return &_iconentries; }

  static ResGroupIcon* findByNameOrOrdinal( const nameorordinal& name );
  QString findBestIcon( int cx, int cy );

private:
  IconHeader _iconHeader;

  QList<ResourceDirectory> _iconentries;

  static QDict<ResGroupIcon> objects;
};

QDataStream& operator>>( QDataStream& s, ResGroupIcon& r );

#endif
