/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klarlvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/**
 * RES file reader.
 *
 * Implementation file for class ResHeader.
 */

#include "ResHeader.h"
#include "tools.h"

QDataStream& operator>>( QDataStream& s, ResHeader& hdr )
{
  s >> hdr.datasize;
  s >> hdr.headersize;
  // read in the type of the resource
  readNameOrOrdinal( s, hdr.restype );

  // read in the name of the resource
  readNameOrOrdinal( s, hdr.resname );

  int nPos = s.device()->at();
  int nPadAlign = sizeof( DWORD );
  int nPad = nPadAlign - (nPos % nPadAlign);
  if( nPadAlign > nPad )
    s.device()->at( nPos + nPad );

  // read in the additional information
  s >> hdr.addinfo.dataversion;
  // PENDING(kalle) Support multiple languages.
  s >> hdr.addinfo.memoryflags;
  s >> hdr.addinfo.languageid;
  s >> hdr.addinfo.version;
  s >> hdr.addinfo.characteristics;

  //  qDebug( "Read in header with name = %s, type = %s, headersize = %d, datasize = %d, dataversion = %d, memoryflags = %d, languageid = %d, version = %d, characteristics = %d", hdr.resname.toString().latin1(), hdr.restype.toString().latin1(), hdr.headersize, hdr.datasize, hdr.addinfo.dataversion, hdr.addinfo.memoryflags, hdr.addinfo.languageid, hdr.addinfo.version, hdr.addinfo.characteristics );
  //  qDebug( "File position at end of header read: %d", s.device()->at() );


  return s;
}


QString ResHeader::toString() const
{
  QString text;
  text = "\tResource header:\n";
  text += "\t\tresource name: ";
  text += resname.toString();
  text += "\n";
  text += "\t\tresource type: ";
  if( !restype.isordinal ) {
	text += restype.string;
  } else {
	switch( restype.id ) {
	case RT_CURSOR:
	  text += "RT_CURSOR";
	  break;
	case RT_BITMAP:
	  text += "RT_BITMAP";
	  break;
	case RT_ICON:
	  text += "RT_ICON";
	  break;
	case RT_MENU:
	  text += "RT_MENU";
	  break;
	case RT_DIALOG:
	  text += "RT_DIALOG";
	  break;
	case RT_STRING:
	  text += "RT_STRING";
	  break;
	case RT_FONTDIR:
	  text += "RT_FONTDIR";
	  break;
	case RT_FONT:
	  text += "RT_FONT";
	  break;
	case RT_ACCELERATORS:
	  text += "RT_ACCELERATORS";
	  break;
	case RT_RCDATA:
	  text += "RT_RCDATA";
	  break;
	case RT_MESSAGETABLE:
	  text += "RT_MESSAGETABLE";
	  break;
	case RT_GROUP_CURSOR:
	  text += "RT_GROUP_CURSOR";
	  break;
	case RT_GROUP_ICON:
	  text += "RT_GROUP_ICON";
	  break;
	case RT_VERSION:
	  text += "RT_VERSION";
	  break;
	case RT_NEWBITMAP:
	  text += "RT_NEWBITMAP";
	  break;
	case RT_NEWMENU:
	  text += "RT_NEWMENU";
	  break;
	case RT_NEWDIALOG:
	  text += "RT_NEWDIALOG";
	  break;
	case RT_DLGINIT:
	  text += "RT_DLGINIT";
	  break;
	case RT_TOOLBAR:
	  text += "RT_TOOLBAR";
	  break;
	}
  }

  text += "\n\n";
  return text;
}
