/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klarlvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/**
 * RES file reader.
 *
 * Header file for class ResHeader.
 */

#ifndef __RESHEADER_H__
#define __RESHEADER_H__

#include "typedefs.h"

#include <qdatastream.h>
#include <qstring.h>

/* Predefined resource types */
#define RT_NEWRESOURCE  0x2000
#define RT_ERROR        0x7fff
#define RT_CURSOR       1
#define RT_BITMAP       2
#define RT_ICON         3
#define RT_MENU         4
#define RT_DIALOG       5
#define RT_STRING       6
#define RT_FONTDIR      7
#define RT_FONT         8
#define RT_ACCELERATORS 9
#define RT_RCDATA       10
#define RT_MESSAGETABLE 11
#define RT_GROUP_CURSOR 12
#define RT_GROUP_ICON   14
#define RT_VERSION      16
#define RT_NEWBITMAP    (RT_BITMAP | RT_NEWRESOURCE)
#define RT_NEWMENU      (RT_MENU | RT_NEWRESOURCE)
#define RT_NEWDIALOG    (RT_DIALOG | RT_NEWRESOURCE)
#define RT_DLGINIT      240 // MFC-specific
#define RT_TOOLBAR      241 // MFC-specific

/* Memory flags used in ResHeader::addinfo.memoryflags */
#define MOVEABLE    0x0010
#define FIXED       ~MOVEABLE
#define PURE        0x0020
#define IMPURE      ~PURE
#define PRELOAD     0x0040
#define LOADONCALL  ~PRELOAD
#define DISCARDABLE 0x1000

class ResHeader
{
  friend QDataStream& operator>>( QDataStream& s, ResHeader& hdr );

public:
  QString toString() const;

  DWORD numericType() const { 
	return restype.isordinal ? restype.id : RT_ERROR; } 
  DWORD dataSize() const { return datasize; }
  nameorordinal name() { return resname; }

private:
  DWORD datasize;
  DWORD headersize;
  nameorordinal restype;
  nameorordinal resname;
  struct tagResource {
	DWORD datasize;
	DWORD headersize;
	nameorordinal restype;
	nameorordinal resname;
	DWORD dataversion;
	WORD memoryflags;
	WORD languageid;
	DWORD version;
	DWORD characteristics;
  } addinfo;
};

QDataStream& operator>>( QDataStream& s, ResHeader& hdr );

#endif
