/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klarlvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/**
 * RES file reader.
 *
 * Implementation file for class ResMenu.
 */

#include "ResMenu.h"
#include "tools.h"

#include <qdatastream.h>

#define GRAYED 0x0001
#define INACTIVE 0x0002
#define BITMAP 0x0004
#define OWNERDRAW 0x0100
#define CHECKED 0x0008
#define POPUP 0x0010
#define MENUBARBREAK 0x0020
#define MENUBREAK 0x0040
#define ENDMENU 0x0080


QString ResMenu::toString() const
{
  QString text;
  text += "\tresource data:\n";
  ResMenuItemListIterator it( itemlist );
  ResMenuItem* current;
  int count = 0;
  while( ( current = it.current() ) ) {
	text += "\t\titem #" + QString::number( count ) + '\n';
	text += "\t\t\tname = " + current->itemtext + '\n';
	count++;
	++it;
  }

  return text;
}


QDataStream& operator>>( QDataStream& s, ResMenu& r )
{
  unsigned int bytesread = 0;
  
  // read MenuHeader
  s >> r.version; bytesread += sizeof( WORD );
  s >> r.headersize; bytesread += sizeof( WORD );

  while( bytesread < r.size ) {
	ResMenuItem* item = new ResMenuItem;
	r.itemlist.append( item );

	// read and assign flags
	WORD itemflags;
	s >> itemflags; bytesread += sizeof( WORD );
	if( itemflags & GRAYED )
	  item->grayed = true;
	else
	  item->grayed = false;
	if( itemflags & INACTIVE )
	  item->inactive = true;
	else
	  item->inactive = false;
	if( itemflags & BITMAP )
	  item->bitmap = true;
	else
	  item->bitmap = false;
	if( itemflags & OWNERDRAW )
	  item->ownerdraw = true;
	else
	  item->ownerdraw = false;
	if( itemflags & CHECKED )
	  item->checked = true;
	else
	  item->checked = false;
	if( itemflags & POPUP )
	  item->popup = true;
	else
	  item->popup = false;
	if( itemflags & MENUBARBREAK )
	  item->menubarbreak = true;
	else
	  item->menubarbreak = false;
	if( itemflags & MENUBREAK )
	  item->menubreak = true;
	else
	  item->menubreak = false;
	if( itemflags & ENDMENU )
	  item->endmenu = true;
	else
	  item->endmenu = false;

	// if this is not a popup, read and assign item id
	if( !item->popup ) {
	  s >> item->itemid; bytesread += sizeof( WORD );
	} else
	  item->itemid = 0;
	
	// read and assign menu text
	readUnicodeString( s, item->itemtext );
	bytesread += item->itemtext.length() * 2 + 2;

	// if name, flags and id are 0, then it is a separator
	if( item->itemtext.isEmpty() && item->itemid == 0 && itemflags == 0 )
	  item->separator = true;
	else
	  item->separator = false;
  }
	
  return s;
}
