/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klarlvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/**
 * $Id: Resource.cpp,v 1.6 2004/03/25 08:46:17 khz Exp $
 *
 * RES file reader.
 *
 * Implementation file for class Resource.
 */

#include "Resource.h"
#include "ResHeader.h"
#include "ResData.h"
#include "ResAccelerators.h"
#include "ResBitmap.h"
#include "ResCursor.h"
#include "ResDialog.h"
#include "ResDlgInit.h"
#include "ResFont.h"
#include "ResFontDir.h"
#include "ResGroupCursor.h"
#include "ResGroupIcon.h"
#include "ResHeader.h"
#include "ResIcon.h"
#include "ResMenu.h"
#include "ResMessageTable.h"
#include "ResNewBitmap.h"
#include "ResNewDialog.h"
#include "ResNewMenu.h"
#include "ResRCData.h"
#include "ResString.h"
#include "ResToolbar.h"
#include "ResUserDefined.h"
#include "ResVersion.h"


// counter of dialogs to be processed in the pre-compiled Knut Evaluation Version
#ifdef KDAB_EVAL
static int N_EVAL_DLG = 0;
#endif


Resource::Resource()
{
  _header = 0;
  _data = 0;
}


Resource::~Resource()
{
  delete _header;
  delete _data;
}


QString Resource::toString() const
{
  return header()->toString() + data()->toString();
}

QDataStream& operator>>( QDataStream& s, Resource& r )
{
// make it a little bit harder to hack our eval code
#ifdef KDAB_EVAL
  const int oldnvldlg = N_EVAL_DLG;
#endif

  // padding needed before we start reading this resource?
  int nPos = s.device()->at();
  int nPadAlign = sizeof( DWORD );
  int nPad = nPadAlign - (nPos % nPadAlign);
  if( nPadAlign > nPad )
    s.device()->at( nPos + nPad );

  //  qDebug( "Start to read header at %d", s.device()->at() );
  // First create and read the resource header
  r._header = new ResHeader;
  s >> *(r._header);
  //  qDebug( "Done reading header at %d", s.device()->at() );

// make it a little bit harder to hack our eval code
#ifdef KDAB_EVAL
  int mxnvldlg = 5000;
  int a =30;
  int b =a+mxnvldlg;
  a+=mxnvldlg;
  const int c =b-10;
  a =10;
  qDebug( "" );
  a = 10+a;
  b = a-8;
  mxnvldlg = (c-a)/1000;
#endif

  // Depending on the type in the header, read the resource data
  if( r.header()->numericType() == RT_ERROR ) {
    qDebug( "Sorry, user-defined resource types are not and will not be supported." );
    int startpos = s.device()->at();

    r._data = new ResUserDefined;
    // But still skip the data.
    //	qDebug( "Skip %ld bytes of data for user-defined resource, file position = %d", r.header()->dataSize(), s.device()->at() );
    uchar c;
    short pad = nPadAlign - ( r.header()->dataSize() % nPadAlign );
    if( pad == nPadAlign )
      pad = 0;
    for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
      s >> c;
/*
    nPos = s.device()->at();
    nPad = nPadAlign - (nPos % nPadAlign);
    if( nPadAlign > nPad )
      s.device()->at( nPos + nPad );
*/
    int endpos = s.device()->at();
    ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
    if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
      qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
      // back to sync
      s.device()->at( startpos + r.header()->dataSize() + pad );
    }
    return s;
  }

  switch( r.header()->numericType() ) {
  case RT_CURSOR: {
	int startpos = s.device()->at();
	r._data = new ResCursor;
	s >> *(ResCursor*)(r._data);
	// skip data for now
	qDebug( "Skip data for RT_CURSOR, file position = %d", s.device()->at() );
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  case RT_BITMAP: {
	int startpos = s.device()->at();
	r._data = new ResBitmap;
	qDebug( "Reading bitmap resource, file position = %d", s.device()->at() );
	s >> *(ResBitmap*)(r._data);
	// skip padding
	if( !s.atEnd() )
	{
		uchar c;
		short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
		if( pad == sizeof( DWORD ) )
		  pad = 0;
		for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
		  s >> c;
		int endpos = s.device()->at();
		ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
		if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) )
		{
		  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d",
				  endpos, startpos, r.header()->dataSize(), pad );
		  // back to sync
		  s.device()->at( startpos + r.header()->dataSize() + pad );
		}
	}
	break;
  }
  case RT_ICON: {
	int startpos = s.device()->at();
	r._data = new ResIcon( r.header() );
	s >> *(ResIcon*)(r._data);
	// skip data for now
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  case RT_MENU: {
	int startpos = s.device()->at();
	r._data = new ResMenu( r.header()->dataSize() );
	qDebug( "Reading menu resource, file position = %d", s.device()->at() );
	s >> *(ResMenu*)(r._data);
	// skip padding
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( int i = 0; i < pad; i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  case RT_DIALOG: {
    r._data = new ResDialog;
#ifdef KDAB_EVAL
    if( N_EVAL_DLG < mxnvldlg )
        ++N_EVAL_DLG;
    else{
        int startpos = s.device()->at();
        uchar c;
        unsigned short pad
            = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
        if( pad == sizeof( DWORD ) )
            pad = 0;
        for( uint i = 0; i < pad; i++ )
            s >> c;
        int endpos = s.device()->at();
        ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
        if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) )
        {
            // back to sync
            s.device()->at( startpos + r.header()->dataSize() + pad );
        }
        break;
    }
#endif

    int startpos = s.device()->at();
    qDebug( "Reading dialog resource, file position = %d", s.device()->at() );
    s >> *(ResDialog*)(r._data);
    // skip padding
    if( !s.atEnd() )
    {
        uchar c;
        unsigned short pad
            = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
        if( pad == sizeof( DWORD ) )
            pad = 0;
        for( uint i = 0; i < pad; i++ )
            s >> c;
        int endpos = s.device()->at();
        ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
        if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) )
        {
            qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d",
                endpos, startpos, r.header()->dataSize(), pad );
            // back to sync
            s.device()->at( startpos + r.header()->dataSize() + pad );
        }
    }
// make it a little bit harder to hack our eval code
#ifdef KDAB_EVAL
    if (N_EVAL_DLG <= oldnvldlg)
        N_EVAL_DLG = oldnvldlg+1;
#endif

    break;
  }
  case RT_STRING: {
	int startpos = s.device()->at();
	qDebug( "Start of RT_STRING %d", startpos );
	r._data = new ResString;
	s >> *(ResString*)(r._data);
	// skip padding
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( int i = 0; i < pad; i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  case RT_FONTDIR: {
	int startpos = s.device()->at();
	r._data = new ResFontDir;
	s >> *(ResFontDir*)(r._data);
	// skip data for now
	qDebug( "Skip data for RT_FONTDIR" );
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  case RT_ACCELERATORS: {
	int startpos = s.device()->at();
	r._data = new ResAccelerators;
	s >> *(ResAccelerators*)(r._data);
	// skip data for now
	qDebug( "Skip data for RT_ACCELERATORS" );
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  case RT_RCDATA: {
	int startpos = s.device()->at();
	r._data = new ResRCData;
	s >> *(ResRCData*)(r._data);
	// skip data for now
	qDebug( "Skip data for RT_RCDATA" );
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  case RT_MESSAGETABLE: {
	int startpos = s.device()->at();
	r._data = new ResMessageTable;
	s >> *(ResMessageTable*)(r._data);
	// skip data for now
	qDebug( "Skip data for RT_MESSAGETABLE" );
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  case RT_GROUP_CURSOR: {
	int startpos = s.device()->at();
	r._data = new ResGroupCursor;
	s >> *(ResGroupCursor*)(r._data);
	// skip data for now
	qDebug( "Skip data for RT_GROUP_CURSOR" );
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  case RT_GROUP_ICON: {
	int startpos = s.device()->at();
	r._data = new ResGroupIcon( r.header() );
	s >> *(ResGroupIcon*)(r._data);
	// skip padding
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( int i = 0; i < pad; i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) )
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  case RT_VERSION: {
	int startpos = s.device()->at();
	r._data = new ResVersion;
	s >> *(ResVersion*)(r._data);
	// skip data for now
	qDebug( "Skip data for RT_VERSION" );
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}

	// PENDING(kalle) Remove
	// Do not read further than version resource
	s.device()->at( s.device()->size() );
	break;
  }
  case RT_NEWBITMAP: {
	int startpos = s.device()->at();
	r._data = new ResNewBitmap;
	s >> *(ResNewBitmap*)(r._data);
	// skip data for now
	qDebug( "Skip data for RT_NEWBITMAP" );
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  case RT_NEWMENU: {
	int startpos = s.device()->at();
	r._data = new ResNewMenu;
	s >> *(ResNewMenu*)(r._data);
	// skip data for now
	qDebug( "Skip data for RT_NEWMENU" );
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  case RT_NEWDIALOG: {
#ifdef KDAB_EVAL
    if( N_EVAL_DLG < mxnvldlg )
        ++N_EVAL_DLG;
    else{
        int startpos = s.device()->at();
        uchar c;
        short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
        if( pad == sizeof( DWORD ) )
            pad = 0;
        for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
            s >> c;
        int endpos = s.device()->at();
        ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
        if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
            // back to sync
            s.device()->at( startpos + r.header()->dataSize() + pad );
        }
        break;
    }
#endif
    int startpos = s.device()->at();
    r._data = new ResNewDialog;
    s >> *(ResNewDialog*)(r._data);
    // skip data for now
    qDebug( "Skip data for RT_NEWDIALOG" );
    uchar c;
    short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
    if( pad == sizeof( DWORD ) )
        pad = 0;
    for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
        s >> c;
    int endpos = s.device()->at();
    ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
    if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
        qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
        // back to sync
        s.device()->at( startpos + r.header()->dataSize() + pad );
    }
// make it a little bit harder to hack our eval code
#ifdef KDAB_EVAL
    if (N_EVAL_DLG <= oldnvldlg)
        N_EVAL_DLG = oldnvldlg+1;
#endif

    break;
  }
  case RT_DLGINIT: {
	int startpos = s.device()->at();
	r._data = new ResDlgInit;
	s >> *(ResDlgInit*)(r._data);
	// skip data for now
	qDebug( "Skip data for RT_DLGINIT" );
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  case RT_TOOLBAR: {
	int startpos = s.device()->at();
	r._data = new ResToolbar;
	s >> *(ResToolbar*)(r._data);
	// skip data for now
	qDebug( "Skip data for RT_TOOLBAR, file position = %d", s.device()->at() );
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
	  s >> c;
	int endpos = s.device()->at();
	ASSERT( ( endpos - startpos ) == ( r.header()->dataSize() + pad ) );
	if( ( endpos - startpos ) != ( r.header()->dataSize() + pad ) ) {
	  qDebug( "endpos = %d, startpos = %d, dataSize = %d, pad = %d", endpos, startpos, r.header()->dataSize(), pad );
	  // back to sync
	  s.device()->at( startpos + r.header()->dataSize() + pad );
	}
	break;
  }
  default: {
	qDebug( "Sorry, user-defined resource types are not and will not be supported." );
	// skip data
	qDebug( "Skip data for user-defined resource" );
	uchar c;
	short pad = sizeof( DWORD ) - ( r.header()->dataSize() % sizeof( DWORD ) );
	if( pad == sizeof( DWORD ) )
	  pad = 0;
	for( uint i = 0; i < ( r.header()->dataSize() + pad ); i++ )
	  s >> c;
  }
  };

// make it a little bit harder to hack our eval code
#ifdef KDAB_EVAL
    if( N_EVAL_DLG > mxnvldlg )
        s.device()->reset();
#endif
  return s;
}
