/* --------------------------------------------------------------------------
 * Copyright (C) 2000-2001 Klarlvdalens Datakonsult AB. All rights reserved.
 *
 * This file is part of KNUT, the Windows resource converter for Qt GUI
 * Designer. You may only use and modify it according to the license terms
 * appearing in the file LICENSE included in this package.
 *
 * Holders of a commercial license have additional rights as specified in the
 * license terms shipped with the commercial package.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * ------------------------------------------------------------------------ */


/**
 * RES file reader.
 *
 * Implementation file for misc functions.
 */

#include "tools.h"
#include "ResHeader.h"

void readNameOrOrdinal( QDataStream& s, nameorordinal& val )
{
  WORD typeorordinal;
  s >> typeorordinal;
  if( typeorordinal == 0xffff ) {
	// type field is ordinal
	val.isordinal = true;
	s >> val.id;
  } else {
	// type field is Unicode string
	val.isordinal = false;
	val.id = RT_ERROR;

	// first build the first short, then read in the remaining
	unsigned short curshort = typeorordinal;
	while( curshort != 0 ) {
	  QChar qc( curshort );
	  val.string += qc;
	  s >> curshort;
	}
  }
}

void readUnicodeString( QDataStream& s, QString& str, bool force )
{
  str = "";

  unsigned short curshort;
  s >> curshort;
  // accept a 0x0000 0x0001 that can be in front of a font string sometimes
  if( curshort == 0 && !force )
	return;
  if( curshort == 0 && force ) {
	// skip 0x0001
	s >> curshort;
	ASSERT( curshort == 256 );
	// now fetch the real one
	s >> curshort;
  }
  while( curshort != 0 ) {
	QChar qc( curshort );
	str += qc;
	s >> curshort;
  }
}


void readUnicodePascalString( QDataStream& s, QString& str )
{
  str = "";

  // read in one word that specifies the number of characters in the string
  Q_UINT16 length;
  s >> length;

  // now read the characters
  for( short i = 0; i < length; i++ ) {
	unsigned short curshort;
	s >> curshort;
	QChar qc( curshort );
	str += qc;
  }

}

