#include <filterinterface.h>

#include <qapplication.h>

#include <qiodevice.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qdir.h>
#include <qprogressdialog.h>

#include "../libresconv/resconv.h"


const char* cProgressCaption  = "Knut";
const char* cProgressProcInit = "initializing...";


class KnutFilter : public ImportFilterInterface, public QLibraryInterface
{
public:
    KnutFilter();

    QRESULT queryInterface( const QUuid&, QUnknownInterface **iface );
    Q_REFCOUNT;

    QStringList featureList() const;
    QStringList import( const QString& filter, const QString& filename );

    bool init();
    void cleanup();
    bool canUnload() const;
};

KnutFilter::KnutFilter()
{
}

QRESULT KnutFilter::queryInterface( const QUuid &uuid, QUnknownInterface **iface )
{
    *iface = 0;
    if ( uuid == IID_QUnknown )
	*iface = (QUnknownInterface*)(ImportFilterInterface*)this;
    else if ( uuid == IID_QFeatureList )
	*iface = (QFeatureListInterface*)this;
    else if ( uuid == IID_ImportFilter )
	*iface = (ImportFilterInterface*)this;
    else if ( uuid == IID_QLibrary )
	*iface = (QLibraryInterface*)this;
    else
	return QE_NOINTERFACE;

    (*iface)->addRef();
    return QS_OK;
}

QStringList KnutFilter::featureList() const
{
    QStringList list;
    list << "Windows EXE/DLL Files (*.exe *.dll)";
    return list;
}

QStringList KnutFilter::import( const QString &, const QString& filename )
{
    QStringList xmlFiles;

    QString home(QDir::home().absPath());
    QFile xmlFilenamesFile( home+"/knutlog" );
    if( !xmlFilenamesFile.open( IO_ReadWrite ) )
        qWarning( "Knut: Could not open process log file." );
    else{
        QFile inputFile( filename );
        if( !inputFile.open( IO_ReadOnly ) )
            qWarning( "Knut: Could not open file '%s' ", filename.latin1() );
        else {
            QProgressDialog* pd
                = new QProgressDialog( cProgressProcInit, 0, 100,
                                       0, cProgressCaption, true
                                       // ,
                                       //   Qt::WStyle_NoBorder
                                       // | Qt::WStyle_Dialog
                                       // | Qt::WStyle_Title*/
                                       );
            pd->setCaption( cProgressCaption );
            pd->setAutoClose( false );
            pd->setAutoReset( false );
            pd->setMinimumDuration( 0 );
            pd->setProgress( 1 );
            pd->show();
            //app.setMainWidget( pd );
            //app.processEvents();
            bool bOk = ResConv::extractToXML( &inputFile,
                                              &xmlFilenamesFile,
                                               1.67,
                                               home+"/.knutrc",
                                               pd );
            pd->setProgress( 100 );
            pd->close();
            if( !bOk ){
                qWarning( "Knut: Could not import resources from file '%s' ", filename.latin1() );
            }else{
                QString xmlFile;
                xmlFilenamesFile.at( 0 );
                QTextStream ts( &xmlFilenamesFile );
                while ( !ts.eof() ){
                    xmlFile = ts.readLine();
                    xmlFiles.append( xmlFile );
                //    printf ("filename %s\n",xmlFile);
                }
            }
        }
        xmlFilenamesFile.close();
        xmlFilenamesFile.remove();
    }
    return xmlFiles;
}

bool KnutFilter::init()
{
    return TRUE;
}

void KnutFilter::cleanup()
{
}

bool KnutFilter::canUnload() const
{
    return TRUE;
}

Q_EXPORT_COMPONENT()
{
    Q_CREATE_INSTANCE( KnutFilter )
}
